// Configured for spectrum shield
//
// For Dual MSGEQ7
// Analogue 0 = Left Channel
// Analogue 1 = Right Channel
// Analogue 2 = LDR
// D4 = Digital O/P Strobe
// D5 = Digital O/P Reset
//
// MAX7219 Control. Assumes two 8x8 LED Matrices
//
// D10 = Digital O/P CS Pin
// D11 = Digital O/P Clock Pin
// D12 = Digital O/P Data Pin
//


// Scrolling text code ripped from...
//
// http://tronixstuff.com/2013/10/11/tutorial-arduino-max7219-led-display-driver-ic/
// Which was...
// based on an orginal sketch by Arduino forum member "danigom"
// http://forum.arduino.cc/index.php?action=profile;u=188950

#include <avr/pgmspace.h>
#include <LedControl.h>


const long scrollDelay = 30;   // adjust scrolling speed
unsigned long bufferLong [14] = {0}; 
const int numDevices = 2;      // number of MAX7219s used
const int dataPin = 12;
const int clkPin = 11;
const int csPin = 10;
LedControl lc=LedControl(dataPin,clkPin,csPin,numDevices);

prog_uchar scrollText1[] PROGMEM ={"MSGEQ7 Demo"};
prog_uchar scrollText2[] PROGMEM ={", By Steve Quinn @        "};

int analogueInputLeftChannel = 0; // read from multiplexer using analog input 0, Left Channel
int analogueInputRightChannel = 1; // read from multiplexer using analog input 1, Right Channel
int strobePin = 4; // strobe is attached to digital pin 4
int resetPin = 5; // reset is attached to digital pin 5
int spectrumValue1[7]; // to hold a2d values
int spectrumValue2[7]; // to hold a2d values

int analogueInputLDR = 5;
byte tempIntensity = 1;

byte tempLeftChannel = 0;
byte tempRightChannel = 0;
unsigned long tempLeftChannela = 0;
unsigned long tempRightChannela = 0;
unsigned long sumLeftChannel = 0;
unsigned long sumRightChannel = 0;
unsigned long leftChannelAverageIndex = 0;
unsigned long rightChannelAverageIndex = 0;
unsigned long rollingAverageLeftChannelOld = 0;
unsigned long rollingAverageRightChannelOld = 0;
unsigned long rollingAverageLeftChannelNew = 0;
unsigned long rollingAverageRightChannelNew = 0;
#define NORMAL_PASS_COUNT 10  // The number off passes through the loop before an average is updated
#define SILENT_PASS_COUNT 30 // How long the dot level persists after audio is removed. Makes the L & R drop slowly after audio is removed.
byte passCount = 0;
byte passCountLimit = NORMAL_PASS_COUNT; 
#define ADC_UPPER_LIMIT 900    // Max upper limit 1023
#define SILENT_CHANNEL_NOISE_SUPPRESSION 150  // stops spurious noise from setting off the display with no audio applied
#define DIVISION_FACTOR 6 // dividing by 4 makes averaging more sensitive
#define LDR_UPPER_LIMIT 1000
#define LDR_LOWER_LIMIT 0

byte movingdot[] = {
      B00000000,
      B10000000,
      B01000000,
      B00100000,
      B00010000,
      B00001000,
      B00000100,
      B00000010,
      B00000001
};

byte bargraph[] = {
      B00000000,
      B10000000,
      B11000000,
      B11100000,
      B11110000,
      B11111000,
      B11111100,
      B11111110,
      B11111111
};


const int leftLEDMatrix = 0;
const int rightLEDMatrix = 1;

void setup(){
   pinMode(analogueInputLeftChannel, INPUT);
   pinMode(analogueInputRightChannel, INPUT);
   pinMode(analogueInputLDR, INPUT);
   pinMode(strobePin, OUTPUT);
   pinMode(resetPin, OUTPUT);
   analogReference(DEFAULT);

   digitalWrite(resetPin, LOW);
   digitalWrite(strobePin, HIGH);
   for (int x=0; x<numDevices; x++){
        lc.shutdown(x,false);       //The MAX72XX is in power-saving mode on startup
        lc.setIntensity(x,tempIntensity);       // Set the brightness to default value
        lc.clearDisplay(x);         // and clear the display
    }
    scrollMessage(scrollText1);
    scrollMessage(scrollText2);
}


void loop(){ 
     tempIntensity = (byte) map(analogRead(analogueInputLDR), LDR_LOWER_LIMIT,LDR_UPPER_LIMIT,15,0);
     for (int x=0; x<numDevices; x++)
       lc.setIntensity(x,tempIntensity);       // Set the brightness from LDR value

     digitalWrite(resetPin, HIGH);
     delayMicroseconds(1); // to allow the output to settle
     digitalWrite(resetPin, LOW);
     delayMicroseconds(72); // to allow the output to settle

     sumLeftChannel = 0;
     sumRightChannel = 0;
     for (int i = 0; i < 7; i++)
     {
       digitalWrite(strobePin, LOW);
       delayMicroseconds(36); // to allow the output to settle
       
       tempLeftChannela = (unsigned long) analogRead(analogueInputLeftChannel);
       if (tempLeftChannela < SILENT_CHANNEL_NOISE_SUPPRESSION) tempLeftChannela = 0;
       tempLeftChannel = (byte) map(tempLeftChannela,0,ADC_UPPER_LIMIT,0,8);
       sumLeftChannel += tempLeftChannel;

       tempRightChannela = (unsigned long) analogRead(analogueInputRightChannel);
       if (tempRightChannela < SILENT_CHANNEL_NOISE_SUPPRESSION) tempRightChannela = 0;
       tempRightChannel = (byte) map(tempRightChannela,0,ADC_UPPER_LIMIT,0,8);
       sumRightChannel += tempRightChannel;
       lc.setRow(rightLEDMatrix,i,bargraph[tempRightChannel]); // In order
       lc.setRow(leftLEDMatrix,7-i,bargraph[tempLeftChannel]); // Reverse order
       digitalWrite(strobePin, HIGH);
       delayMicroseconds(36); // to allow the output to settle
     }
          
      // Detects when audio is not present and increases the number of passes in the loop before the moving dot is dropped
      if ((rollingAverageLeftChannelOld == rollingAverageLeftChannelNew) &&
          (rollingAverageRightChannelOld == rollingAverageRightChannelNew) &&
          (rollingAverageLeftChannelNew == 0) &&
          (rollingAverageRightChannelNew == 0))
        passCountLimit = SILENT_PASS_COUNT;
      else
        passCountLimit = NORMAL_PASS_COUNT;

     if (passCount > passCountLimit) 
     {
         passCount = 0;

         rollingAverageLeftChannelNew = map(sumLeftChannel/DIVISION_FACTOR,0,8,0,8);

         if ((rollingAverageLeftChannelNew == rollingAverageLeftChannelOld) && 
             (rollingAverageLeftChannelNew == 0) && 
             (leftChannelAverageIndex > 0))
         {
            if (leftChannelAverageIndex > 0) leftChannelAverageIndex--;
            lc.setRow(leftLEDMatrix,0,movingdot[leftChannelAverageIndex]); 
         }

         if (rollingAverageLeftChannelOld != rollingAverageLeftChannelNew) 
         {
            if (rollingAverageLeftChannelOld < rollingAverageLeftChannelNew) 
            {
              if (leftChannelAverageIndex < 8) leftChannelAverageIndex++;
            } else {
                 if (rollingAverageLeftChannelOld > rollingAverageLeftChannelNew) 
                 {
                   if (leftChannelAverageIndex > 0) leftChannelAverageIndex--;
                 } else {
                   // The same do nothing
                 }
            }
            lc.setRow(leftLEDMatrix,0,movingdot[leftChannelAverageIndex]); 
            rollingAverageLeftChannelOld = rollingAverageLeftChannelNew;
         }

         rollingAverageRightChannelNew = map(sumRightChannel/DIVISION_FACTOR,0,8,0,8);

         if ((rollingAverageRightChannelNew == rollingAverageRightChannelOld) && 
             (rollingAverageRightChannelNew == 0) && 
             (rightChannelAverageIndex > 0))
         {
            if (rightChannelAverageIndex > 0) rightChannelAverageIndex--;
            lc.setRow(rightLEDMatrix,7,movingdot[rightChannelAverageIndex]); 
         }

         if (rollingAverageRightChannelOld != rollingAverageRightChannelNew) 
         {
            if (rollingAverageRightChannelOld < rollingAverageRightChannelNew) 
            {
              if (rightChannelAverageIndex < 8) rightChannelAverageIndex++;
            } else {
                 if (rollingAverageRightChannelOld > rollingAverageRightChannelNew) 
                 {
                    if (rightChannelAverageIndex > 0) rightChannelAverageIndex--;
                 } else {
                     // The same do nothing
                 }
            }
            lc.setRow(rightLEDMatrix,7,movingdot[rightChannelAverageIndex]); 
            rollingAverageRightChannelOld = rollingAverageRightChannelNew;
         }
     } else {
         passCount++;     
     }
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

prog_uchar font5x7 [] PROGMEM = {      //Numeric Font Matrix (Arranged as 7x font data + 1x kerning data)
    B00000000,	//Space (Char 0x20)
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    6,

    B10000000,	//!
    B10000000,
    B10000000,
    B10000000,
    B00000000,
    B00000000,
    B10000000,
    2,

    B10100000,	//"
    B10100000,
    B10100000,
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    4,

    B01010000,	//#
    B01010000,
    B11111000,
    B01010000,
    B11111000,
    B01010000,
    B01010000,
    6,

    B00100000,	//$
    B01111000,
    B10100000,
    B01110000,
    B00101000,
    B11110000,
    B00100000,
    6,

    B11000000,	//%
    B11001000,
    B00010000,
    B00100000,
    B01000000,
    B10011000,
    B00011000,
    6,

    B01100000,	//&
    B10010000,
    B10100000,
    B01000000,
    B10101000,
    B10010000,
    B01101000,
    6,

    B11000000,	//'
    B01000000,
    B10000000,
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    3,

    B00100000,	//(
    B01000000,
    B10000000,
    B10000000,
    B10000000,
    B01000000,
    B00100000,
    4,

    B10000000,	//)
    B01000000,
    B00100000,
    B00100000,
    B00100000,
    B01000000,
    B10000000,
    4,

    B00000000,	//*
    B00100000,
    B10101000,
    B01110000,
    B10101000,
    B00100000,
    B00000000,
    6,

    B00000000,	//+
    B00100000,
    B00100000,
    B11111000,
    B00100000,
    B00100000,
    B00000000,
    6,

    B00000000,	//,
    B00000000,
    B00000000,
    B00000000,
    B11000000,
    B01000000,
    B10000000,
    3,

    B00000000,	//-
    B00000000,
    B11111000,
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    6,

    B00000000,	//.
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    B11000000,
    B11000000,
    3,

    B00000000,	///
    B00001000,
    B00010000,
    B00100000,
    B01000000,
    B10000000,
    B00000000,
    6,

    B01110000,	//0
    B10001000,
    B10011000,
    B10101000,
    B11001000,
    B10001000,
    B01110000,
    6,

    B01000000,	//1
    B11000000,
    B01000000,
    B01000000,
    B01000000,
    B01000000,
    B11100000,
    4,

    B01110000,	//2
    B10001000,
    B00001000,
    B00010000,
    B00100000,
    B01000000,
    B11111000,
    6,

    B11111000,	//3
    B00010000,
    B00100000,
    B00010000,
    B00001000,
    B10001000,
    B01110000,
    6,

    B00010000,	//4
    B00110000,
    B01010000,
    B10010000,
    B11111000,
    B00010000,
    B00010000,
    6,

    B11111000,	//5
    B10000000,
    B11110000,
    B00001000,
    B00001000,
    B10001000,
    B01110000,
    6,

    B00110000,	//6
    B01000000,
    B10000000,
    B11110000,
    B10001000,
    B10001000,
    B01110000,
    6,

    B11111000,	//7
    B10001000,
    B00001000,
    B00010000,
    B00100000,
    B00100000,
    B00100000,
    6,

    B01110000,	//8
    B10001000,
    B10001000,
    B01110000,
    B10001000,
    B10001000,
    B01110000,
    6,

    B01110000,	//9
    B10001000,
    B10001000,
    B01111000,
    B00001000,
    B00010000,
    B01100000,
    6,

    B00000000,	//:
    B11000000,
    B11000000,
    B00000000,
    B11000000,
    B11000000,
    B00000000,
    3,

    B00000000,	//;
    B11000000,
    B11000000,
    B00000000,
    B11000000,
    B01000000,
    B10000000,
    3,

    B00010000,	//<
    B00100000,
    B01000000,
    B10000000,
    B01000000,
    B00100000,
    B00010000,
    5,

    B00000000,	//=
    B00000000,
    B11111000,
    B00000000,
    B11111000,
    B00000000,
    B00000000,
    6,

    B10000000,	//>
    B01000000,
    B00100000,
    B00010000,
    B00100000,
    B01000000,
    B10000000,
    5,

    B01110000,	//?
    B10001000,
    B00001000,
    B00010000,
    B00100000,
    B00000000,
    B00100000,
    6,

    B01111110,	//@
    B10000001,
    B10100101,
    B10000001,
    B11000011,
    B10111101,
    B01111110,
    8,

    B01110000,	//A
    B10001000,
    B10001000,
    B10001000,
    B11111000,
    B10001000,
    B10001000,
    6,

    B11110000,	//B
    B10001000,
    B10001000,
    B11110000,
    B10001000,
    B10001000,
    B11110000,
    6,

    B01110000,	//C
    B10001000,
    B10000000,
    B10000000,
    B10000000,
    B10001000,
    B01110000,
    6,

    B11100000,	//D
    B10010000,
    B10001000,
    B10001000,
    B10001000,
    B10010000,
    B11100000,
    6,

    B11111000,	//E
    B10000000,
    B10000000,
    B11110000,
    B10000000,
    B10000000,
    B11111000,
    6,

    B11111000,	//F
    B10000000,
    B10000000,
    B11110000,
    B10000000,
    B10000000,
    B10000000,
    6,

    B01110000,	//G
    B10001000,
    B10000000,
    B10111000,
    B10001000,
    B10001000,
    B01111000,
    6,

    B10001000,	//H
    B10001000,
    B10001000,
    B11111000,
    B10001000,
    B10001000,
    B10001000,
    6,

    B11100000,	//I
    B01000000,
    B01000000,
    B01000000,
    B01000000,
    B01000000,
    B11100000,
    4,

    B00111000,	//J
    B00010000,
    B00010000,
    B00010000,
    B00010000,
    B10010000,
    B01100000,
    6,

    B10001000,	//K
    B10010000,
    B10100000,
    B11000000,
    B10100000,
    B10010000,
    B10001000,
    6,

    B10000000,	//L
    B10000000,
    B10000000,
    B10000000,
    B10000000,
    B10000000,
    B11111000,
    6,

    B10001000,	//M
    B11011000,
    B10101000,
    B10101000,
    B10001000,
    B10001000,
    B10001000,
    6,

    B10001000,	//N
    B10001000,
    B11001000,
    B10101000,
    B10011000,
    B10001000,
    B10001000,
    6,

    B01110000,	//O
    B10001000,
    B10001000,
    B10001000,
    B10001000,
    B10001000,
    B01110000,
    6,

    B11110000,	//P
    B10001000,
    B10001000,
    B11110000,
    B10000000,
    B10000000,
    B10000000,
    6,

    B01110000,	//Q
    B10001000,
    B10001000,
    B10001000,
    B10101000,
    B10010000,
    B01101000,
    6,

    B11110000,	//R
    B10001000,
    B10001000,
    B11110000,
    B10100000,
    B10010000,
    B10001000,
    6,

    B01111000,	//S
    B10000000,
    B10000000,
    B01110000,
    B00001000,
    B00001000,
    B11110000,
    6,

    B11111000,	//T
    B00100000,
    B00100000,
    B00100000,
    B00100000,
    B00100000,
    B00100000,
    6,

    B10001000,	//U
    B10001000,
    B10001000,
    B10001000,
    B10001000,
    B10001000,
    B01110000,
    6,

    B10001000,	//V
    B10001000,
    B10001000,
    B10001000,
    B10001000,
    B01010000,
    B00100000,
    6,

    B10001000,	//W
    B10001000,
    B10001000,
    B10101000,
    B10101000,
    B10101000,
    B01010000,
    6,

    B10001000,	//X
    B10001000,
    B01010000,
    B00100000,
    B01010000,
    B10001000,
    B10001000,
    6,

    B10001000,	//Y
    B10001000,
    B10001000,
    B01010000,
    B00100000,
    B00100000,
    B00100000,
    6,

    B11111000,	//Z
    B00001000,
    B00010000,
    B00100000,
    B01000000,
    B10000000,
    B11111000,
    6,

    B11100000,	//[
    B10000000,
    B10000000,
    B10000000,
    B10000000,
    B10000000,
    B11100000,
    4,

    B00000000,	//(Backward Slash)
    B10000000,
    B01000000,
    B00100000,
    B00010000,
    B00001000,
    B00000000,
    6,

    B11100000,	//]
    B00100000,
    B00100000,
    B00100000,
    B00100000,
    B00100000,
    B11100000,
    4,

    B00100000,	//^
    B01010000,
    B10001000,
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    6,

    B00000000,	//_
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    B11111000,
    6,

    B10000000,	//`
    B01000000,
    B00100000,
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    4,

    B00000000,	//a
    B00000000,
    B01110000,
    B00001000,
    B01111000,
    B10001000,
    B01111000,
    6,

    B10000000,	//b
    B10000000,
    B10110000,
    B11001000,
    B10001000,
    B10001000,
    B11110000,
    6,

    B00000000,	//c
    B00000000,
    B01110000,
    B10001000,
    B10000000,
    B10001000,
    B01110000,
    6,

    B00001000,	//d
    B00001000,
    B01101000,
    B10011000,
    B10001000,
    B10001000,
    B01111000,
    6,

    B00000000,	//e
    B00000000,
    B01110000,
    B10001000,
    B11111000,
    B10000000,
    B01110000,
    6,

    B00110000,	//f
    B01001000,
    B01000000,
    B11100000,
    B01000000,
    B01000000,
    B01000000,
    6,

    B00000000,	//g
    B01111000,
    B10001000,
    B10001000,
    B01111000,
    B00001000,
    B01110000,
    6,

    B10000000,	//h
    B10000000,
    B10110000,
    B11001000,
    B10001000,
    B10001000,
    B10001000,
    6,

    B01000000,	//i
    B00000000,
    B11000000,
    B01000000,
    B01000000,
    B01000000,
    B11100000,
    4,

    B00010000,	//j
    B00000000,
    B00110000,
    B00010000,
    B00010000,
    B10010000,
    B01100000,
    5,

    B10000000,	//k
    B10000000,
    B10010000,
    B10100000,
    B11000000,
    B10100000,
    B10010000,
    5,

    B11000000,	//l
    B01000000,
    B01000000,
    B01000000,
    B01000000,
    B01000000,
    B11100000,
    4,

    B00000000,	//m
    B00000000,
    B11010000,
    B10101000,
    B10101000,
    B10001000,
    B10001000,
    6,

    B00000000,	//n
    B00000000,
    B10110000,
    B11001000,
    B10001000,
    B10001000,
    B10001000,
    6,

    B00000000,	//o
    B00000000,
    B01110000,
    B10001000,
    B10001000,
    B10001000,
    B01110000,
    6,

    B00000000,	//p
    B00000000,
    B11110000,
    B10001000,
    B11110000,
    B10000000,
    B10000000,
    6,

    B00000000,	//q
    B00000000,
    B01101000,
    B10011000,
    B01111000,
    B00001000,
    B00001000,
    6,

    B00000000,	//r
    B00000000,
    B10110000,
    B11001000,
    B10000000,
    B10000000,
    B10000000,
    6,

    B00000000,	//s
    B00000000,
    B01110000,
    B10000000,
    B01110000,
    B00001000,
    B11110000,
    6,

    B01000000,	//t
    B01000000,
    B11100000,
    B01000000,
    B01000000,
    B01001000,
    B00110000,
    6,

    B00000000,	//u
    B00000000,
    B10001000,
    B10001000,
    B10001000,
    B10011000,
    B01101000,
    6,

    B00000000,	//v
    B00000000,
    B10001000,
    B10001000,
    B10001000,
    B01010000,
    B00100000,
    6,

    B00000000,	//w
    B00000000,
    B10001000,
    B10101000,
    B10101000,
    B10101000,
    B01010000,
    6,

    B00000000,	//x
    B00000000,
    B10001000,
    B01010000,
    B00100000,
    B01010000,
    B10001000,
    6,

    B00000000,	//y
    B00000000,
    B10001000,
    B10001000,
    B01111000,
    B00001000,
    B01110000,
    6,

    B00000000,	//z
    B00000000,
    B11111000,
    B00010000,
    B00100000,
    B01000000,
    B11111000,
    6,

    B00100000,	//{
    B01000000,
    B01000000,
    B10000000,
    B01000000,
    B01000000,
    B00100000,
    4,

    B10000000,	//|
    B10000000,
    B10000000,
    B10000000,
    B10000000,
    B10000000,
    B10000000,
    2,

    B10000000,	//}
    B01000000,
    B01000000,
    B00100000,
    B01000000,
    B01000000,
    B10000000,
    4,

    B00000000,	//~
    B00000000,
    B00000000,
    B01101000,
    B10010000,
    B00000000,
    B00000000,
    6,

    B01100000,	// (Char 0x7F)
    B10010000,
    B10010000,
    B01100000,
    B00000000,
    B00000000,
    B00000000,
    5
};

void scrollFont() {
    for (int counter=0x20;counter<0x80;counter++){
        loadBufferLong(counter);
        delay(500);
    }
}

// Scroll Message
void scrollMessage(prog_uchar * messageString) {
    int counter = 0;
    int myChar=0;
    do {
        // read back a char 
        myChar =  pgm_read_byte_near(messageString + counter); 
        if (myChar != 0){
            loadBufferLong(myChar);
        }
        counter++;
    } 
    while (myChar != 0);
}
// Load character into scroll buffer
void loadBufferLong(int ascii){
    if (ascii >= 0x20 && ascii <=0x7f){
        for (int a=0;a<7;a++){                      // Loop 7 times for a 5x7 font
            unsigned long c = pgm_read_byte_near(font5x7 + ((ascii - 0x20) * 8) + a);     // Index into character table to get row data
            unsigned long x = bufferLong [a*2];     // Load current scroll buffer
            x = x | c;                              // OR the new character onto end of current
            bufferLong [a*2] = x;                   // Store in buffer
        }
        byte count = pgm_read_byte_near(font5x7 +((ascii - 0x20) * 8) + 7);     // Index into character table for kerning data
        for (byte x=0; x<count;x++){
            rotateBufferLong();
            printBufferLong();
            delay(scrollDelay);
        }
    }
}
// Rotate the buffer
void rotateBufferLong(){
    for (int a=0;a<7;a++){                      // Loop 7 times for a 5x7 font
        unsigned long x = bufferLong [a*2];     // Get low buffer entry
        byte b = bitRead(x,31);                 // Copy high order bit that gets lost in rotation
        x = x<<1;                               // Rotate left one bit
        bufferLong [a*2] = x;                   // Store new low buffer
        x = bufferLong [a*2+1];                 // Get high buffer entry
        x = x<<1;                               // Rotate left one bit
        bitWrite(x,0,b);                        // Store saved bit
        bufferLong [a*2+1] = x;                 // Store new high buffer
    }
}  
// Display Buffer on LED matrix
void printBufferLong(){
  for (int a=0;a<7;a++){                    // Loop 7 times for a 5x7 font
    unsigned long x = bufferLong [a*2+1];   // Get high buffer entry
    byte y = x;                             // Mask off first character
//    lc.setRow(3,a,y);                       // Send row to relevent MAX7219 chip
    lc.setColumn(0,7-a,y);                       // Send row to relevent MAX7219 chip
    x = bufferLong [a*2];                   // Get low buffer entry
    y = (x>>24);                            // Mask off second character
//    lc.setRow(2,a,y);                       // Send row to relevent MAX7219 chip
    lc.setColumn(1,7-a,y);                       // Send row to relevent MAX7219 chip
    y = (x>>16);                            // Mask off third character
//    lc.setRow(1,a,y);                       // Send row to relevent MAX7219 chip
    lc.setColumn(2,7-a,y);                       // Send row to relevent MAX7219 chip
    y = (x>>8);                             // Mask off forth character
//    lc.setRow(0,a,y);                       // Send row to relevent MAX7219 chip
    lc.setColumn(3,7-a,y);                       // Send row to relevent MAX7219 chip
  }
}

